/*
 *  Copyright 2005 Youjin Lee
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package net.kldp.whitebear.common.config;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Properties;

import javax.swing.table.DefaultTableModel;

import net.kldp.whitebear.common.entity.DbConnectInfo;
import net.kldp.whitebear.common.log.Log;
import net.kldp.whitebear.common.variable.DBTypes;
import net.kldp.whitebear.common.variable.Messages;
import net.kldp.whitebear.handler.StartUpHandler;
import net.kldp.whitebear.handler.ui.swing.ReadOnlyTable;



public class SessionInfoConfig {
    private final String PREFIX     = "session";
    private final String SEPERATOR  = ".";
    private final String ALLOC_STR  = "=";
    private final String SIZE       = SEPERATOR+"size";
    private final String DBTYPENAME = SEPERATOR+"dbTypeName";
    private final String DBNAME     = SEPERATOR+"dbName";
    private final String HOST       = SEPERATOR+"host";
    private final String PORT       = SEPERATOR+"port";
    private final String USERNAME   = SEPERATOR+"username";
    private final String PASSWORD   = SEPERATOR+"password";
    
    private final String sessionInfoFileName = "./conf/sessioninfo.properties";
    private final String backupInfoFileName = "./conf/sessioninfo.properties.bak";
    private Properties sessionInfo;

    private Properties getSessionInfo() {
        if (sessionInfo==null) {
            sessionInfo = new Properties();
            try {
                InputStream inputStream = new FileInputStream(sessionInfoFileName);
                sessionInfo.load(inputStream);
                inputStream.close();
            } catch (IOException ioe) {
                Log.logger.error(Messages.LOG_MSG_00016, ioe);
                return null;
            }            
        }
        return sessionInfo;
    }
    
    public ArrayList getDbConnectInfoList() {
        ArrayList list = null;
        int size = getSessionSize();

        for (int i=0; i<size; i++) {
            DbConnectInfo dbConnectInfo = getDbConnectInfo(i);
            if (dbConnectInfo!=null) {
                if (list==null) {
                    list = new ArrayList();
                }
                list.add(dbConnectInfo);
            }
        }
        return list;
    }
    
    public DbConnectInfo getDbConnectInfo(int position) {
        DbConnectInfo dbConnectInfo = null;
        
        String dbTypeName = sessionInfo.get(PREFIX+position+DBTYPENAME).toString().trim();
        String dbName     = sessionInfo.get(PREFIX+position+DBNAME).toString().trim();
        String host       = sessionInfo.get(PREFIX+position+HOST).toString().trim();
        String port       = sessionInfo.get(PREFIX+position+PORT).toString().trim();
        String username   = sessionInfo.get(PREFIX+position+USERNAME).toString().trim();
        String password   = sessionInfo.get(PREFIX+position+PASSWORD).toString().trim();

        try {
            int iDbtype = DBTypes.getDbType(dbTypeName);
            if (DBTypes.isCorrectDbType(iDbtype)) {
                dbConnectInfo = new DbConnectInfo();
                dbConnectInfo.setDbtype(iDbtype);    
            }
        } catch (NumberFormatException nfe) {
            Log.logger.error(Messages.LOG_MSG_00018, nfe);
            return null;
        }            

        dbConnectInfo.setDbName(dbName);
        dbConnectInfo.setHost(host);
        dbConnectInfo.setPort(port);
        dbConnectInfo.setUserName(username);
        dbConnectInfo.setPassword(password);
        return dbConnectInfo;
    }
    
    private int getSessionSize() {
        int size = 0;
        try {
            size = Integer.parseInt(getSessionInfo().get(PREFIX+SIZE).toString().trim());
        } catch (NumberFormatException nfe) {
            Log.logger.error(Messages.LOG_MSG_00016, nfe);
        }
        return size;
    }
    
    private boolean backupSessionInfoFile() {
        File sessionInfoFile = new File(sessionInfoFileName);
        File backupFile = new File(backupInfoFileName);
        backupFile.delete();
        boolean isRenameSucess = sessionInfoFile.renameTo(backupFile);
        return isRenameSucess;
    }
    
    private DbConnectInfo getDbConnectInfo(ReadOnlyTable readOnlyTable, int pos) {
        DbConnectInfo dbConnectInfo = null;
        dbConnectInfo = new DbConnectInfo();
        dbConnectInfo.setDbName  ((String)readOnlyTable.getValueAt(pos,StartUpHandler.TABLE_HEADER_DB_NAME));
        dbConnectInfo.setHost    ((String)readOnlyTable.getValueAt(pos,StartUpHandler.TABLE_HEADER_DB_HOST));
        dbConnectInfo.setPort    ((String)readOnlyTable.getValueAt(pos,StartUpHandler.TABLE_HEADER_DB_PORT));
        dbConnectInfo.setUserName((String)readOnlyTable.getValueAt(pos,StartUpHandler.TABLE_HEADER_DB_USERNAME));
        dbConnectInfo.setDbtype  (DBTypes.getDbType((String)readOnlyTable.getValueAt(pos,StartUpHandler.TABLE_HEADER_DB_TYPE)));
        return dbConnectInfo; 
    }
    
    private ArrayList getDbConnectInfoList(ReadOnlyTable readOnlyTable) {
        ArrayList list = null;
        if (readOnlyTable.getModel() instanceof DefaultTableModel) {
            list = new ArrayList();
            for (int i=0; i<readOnlyTable.getRowCount(); i++) {
                DbConnectInfo dbConnectInfo = getDbConnectInfo(readOnlyTable, i);
                list.add(dbConnectInfo);
            }
        }
        return list;         
    }
    
    public synchronized void store(ReadOnlyTable readOnlyTable) {
        ArrayList list = getDbConnectInfoList(readOnlyTable);
        
        if (list!=null && list.size()>0 ) {

            if (backupSessionInfoFile()==false) {
                Log.logger.error(Messages.LOG_MSG_00020);
            }
            
            try {
                FileWriter fileWriter = new FileWriter(sessionInfoFileName);
                BufferedWriter bufferedWriter = new BufferedWriter(fileWriter);

                bufferedWriter.write(PREFIX+SIZE+ALLOC_STR);
                bufferedWriter.write(Integer.toString(list.size())+"\n");
                bufferedWriter.write("\n");
                
                for (int i=0; i<list.size(); i++) {
                    DbConnectInfo dbConnectInfo = (DbConnectInfo)list.get(i);
                    bufferedWriter.write(PREFIX+i+DBTYPENAME+ALLOC_STR);
                    bufferedWriter.write(DBTypes.getDbTypeName(dbConnectInfo.getDbtype())+"\n");
                    bufferedWriter.write(PREFIX+i+DBNAME+ALLOC_STR);
                    bufferedWriter.write(dbConnectInfo.getDbName()+"\n");
                    bufferedWriter.write(PREFIX+i+HOST+ALLOC_STR);
                    bufferedWriter.write(dbConnectInfo.getHost()+"\n");
                    bufferedWriter.write(PREFIX+i+PORT+ALLOC_STR);
                    bufferedWriter.write(dbConnectInfo.getPort()+"\n");
                    bufferedWriter.write(PREFIX+i+USERNAME+ALLOC_STR);
                    bufferedWriter.write(dbConnectInfo.getUserName()+"\n");
                    bufferedWriter.write(PREFIX+i+PASSWORD+ALLOC_STR);
                    String password = dbConnectInfo.getPassword();
                    if (password==null) {
                        password = "";
                    }
                    bufferedWriter.write(password+"\n");
                    bufferedWriter.write("\n");
                }
                
                bufferedWriter.close();
                fileWriter.close();
                
            } catch (IOException ioe) {
                Log.logger.error(Messages.LOG_MSG_00019, ioe);
            }
        }
    }
    
}


