/*
 *  Copyright 2005 Youjin Lee
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package net.kldp.whitebear.handler;


import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import net.kldp.whitebear.common.entity.ColumnRef;
import net.kldp.whitebear.common.entity.DatabaseInfo;
import net.kldp.whitebear.common.entity.DbConnectInfo;
import net.kldp.whitebear.common.entity.TableRef;
import net.kldp.whitebear.common.log.Log;
import net.kldp.whitebear.common.variable.DBTypes;
import net.kldp.whitebear.common.variable.Messages;



public class JdbcHandler {
    private int maxReConnectionSize=2;
    private int maxTableMapSize=100;
    private int maxColumnMapSize=100;
    
    private Connection connection;
    private DbConnectInfo dbConnectInfo;
    private DatabaseInfo databaseInfo;
    private ArrayList categories;
    private Map tableMap;
    private Map columnMap;
    
    public JdbcHandler(DbConnectInfo dbConnectInfo) throws SQLException {
        this.dbConnectInfo = dbConnectInfo;
        dbConnectInfo.setUrl(makeUrl(dbConnectInfo));
        connection = getConnection(dbConnectInfo);
        if (connection==null) {
            reConnect();
            if (connection==null) {
                throw new SQLException(Messages.LOG_MSG_00015); 
            }
        }
        tableMap = Collections.synchronizedMap(new HashMap());
        columnMap = Collections.synchronizedMap(new HashMap());
    }
    
    public Connection getConnection() {
        if (isConnectionAvailable()) {
            return connection;
        }
        
        reConnect();
        return connection;
    }    
    
    private void reConnect() {
        if (isConnectionAvailable()==false) {
            Log.logger.error(Messages.LOG_MSG_00006);
            connection = null;
            for (int i=0; i<maxReConnectionSize; i++) {
                connection = getConnection(dbConnectInfo);
                if (isConnectionAvailable()==true) {
                    return;
                }
            }
            Log.logger.error(Messages.LOG_MSG_00008);
        }
    }
    
    private boolean isConnectionAvailable() {
        boolean isAvailable = false;
        
        if (connection!=null) {
            try {
                if (connection.isClosed()==false){
                    isAvailable = true;
                }
            } catch (SQLException sqle) {
                Log.logger.error(Messages.LOG_MSG_00013, sqle);
            }
        }
        
        return isAvailable;
    }
    
    public void close() {
        try {
            if (connection!=null) { connection.close(); }
            if (dbConnectInfo!=null) { dbConnectInfo=null; }
            if (databaseInfo!=null) { databaseInfo=null; }
            if (categories!=null) { categories=null; }
            if (tableMap!=null) { tableMap.clear(); }
            if (columnMap!=null) { columnMap.clear(); }
        } catch (SQLException sqle) {
            Log.logger.error(Messages.LOG_MSG_00009, sqle);
        }
    }    
    
    public Connection getConnection(DbConnectInfo dbConnectInfo) {
        Connection con = null;
        try {
            switch (dbConnectInfo.getDbtype()) {
            case DBTypes.DB_TYPE_ORACLE:
                DriverManager.registerDriver(new oracle.jdbc.OracleDriver());
                con = DriverManager.getConnection(dbConnectInfo.getUrl(), dbConnectInfo.getUserName(), dbConnectInfo.getPassword());
                break;
            case DBTypes.DB_TYPE_MYSQL:
                Class.forName("com.mysql.jdbc.Driver");
                con = DriverManager.getConnection(dbConnectInfo.getUrl(), dbConnectInfo.getUserName(), dbConnectInfo.getPassword());
                break;
            default:
                Log.logger.error(Messages.LOG_MSG_00002);
                break;
            }
        } catch (SQLException sqle) {
            Log.logger.error(Messages.LOG_MSG_00001, sqle);
            return null;
        } catch (ClassNotFoundException cnfe) {
            Log.logger.error(Messages.LOG_MSG_00014, cnfe);
            return null;
        }
        return con;
    }

    public ArrayList getCategories(DatabaseMetaData dbMeta) {
        if (categories==null) {
            ResultSet rs = null;
            String fieldName = "";
            try {
                switch (dbConnectInfo.getDbtype()) {
                case DBTypes.DB_TYPE_ORACLE:
                    rs = dbMeta.getSchemas();
                    fieldName = "TABLE_SCHEM";
                    break;
                case DBTypes.DB_TYPE_MYSQL:
                    rs = dbMeta.getCatalogs();
                    fieldName = "TABLE_CAT";
                    break;
                }
                
                while (rs.next()) {
                    if (categories==null) {
                        categories = new ArrayList();
                    }
                    String category = rs.getString(fieldName);
                    categories.add(category);
                }
            } catch (SQLException sqle) {
                Log.logger.error(Messages.LOG_MSG_00004, sqle);            
            }
        }

        return categories;
    }
    
    public ArrayList getTables(DatabaseMetaData dbMeta, String category) {
        if (tableMap.containsKey(category)) {
            return (ArrayList)tableMap.get(category);
        } else {
            ArrayList tables = null;
            ResultSet rs = null;
            try {
                switch (dbConnectInfo.getDbtype()) {
                case DBTypes.DB_TYPE_ORACLE:
                    rs = dbMeta.getTables(null, category, null, null);
                    break;
                case DBTypes.DB_TYPE_MYSQL:
                    rs = dbMeta.getTables(category, null, null, null);
                    break;
                } 
                
                while (rs.next()) {
                    if (tables==null) {
                        tables = new ArrayList();
                    }
                    
                    TableRef t = new TableRef();
                    try { t.setTABLE_CAT                (rs.getString("TABLE_CAT"                )); } catch (SQLException sqle) {}
                    try { t.setTABLE_SCHEM              (rs.getString("TABLE_SCHEM"              )); } catch (SQLException sqle) {}
                    try { t.setTABLE_NAME               (rs.getString("TABLE_NAME"               )); } catch (SQLException sqle) {}
                    try { t.setTABLE_TYPE               (rs.getString("TABLE_TYPE"               )); } catch (SQLException sqle) {}
                    try { t.setREMARKS                  (rs.getString("REMARKS"                  )); } catch (SQLException sqle) {}
                    try { t.setTYPE_CAT                 (rs.getString("TYPE_CAT"                 )); } catch (SQLException sqle) {}
                    try { t.setTYPE_SCHEM               (rs.getString("TYPE_SCHEM"               )); } catch (SQLException sqle) {}
                    try { t.setTYPE_NAME                (rs.getString("TYPE_NAME"                )); } catch (SQLException sqle) {}
                    try { t.setSELF_REFERENCING_COL_NAME(rs.getString("SELF_REFERENCING_COL_NAME")); } catch (SQLException sqle) {}
                    try { t.setREF_GENERATION           (rs.getString("REF_GENERATION"           )); } catch (SQLException sqle) {}
                    tables.add(t);
                }
            } catch (SQLException sqle) {
                Log.logger.error(Messages.LOG_MSG_00005, sqle);
            }
            if (tableMap.size() < maxTableMapSize){
                tableMap.put(category, tables);
            }
            return tables;            
        }

    }
    
    public ArrayList getColumns(DatabaseMetaData dbMeta, String category, String table) {
        String key = category+","+table;
        if (columnMap.containsKey(key)) {
            return (ArrayList)columnMap.get(key);
        } else {
            ArrayList columns = null;
            ResultSet columnRs = null;
            try {
                switch (dbConnectInfo.getDbtype()) {
                case DBTypes.DB_TYPE_ORACLE:
                    columnRs = dbMeta.getColumns(null, category, table, null);
                    break;
                case DBTypes.DB_TYPE_MYSQL:
                    columnRs = dbMeta.getColumns(category, null, table, null);
                    break;
                }             
                
                while (columnRs.next()) {
                    if (columns == null) {
                        columns = new ArrayList();
                    }
                    
                    ColumnRef c = new ColumnRef();
                    try { c.setTABLE_CAT        (columnRs.getString("TABLE_CAT")        ); } catch (SQLException sqle) {}
                    try { c.setTABLE_SCHEM      (columnRs.getString("TABLE_SCHEM")      ); } catch (SQLException sqle) {}
                    try { c.setTABLE_NAME       (columnRs.getString("TABLE_NAME")       ); } catch (SQLException sqle) {}
                    try { c.setCOLUMN_NAME      (columnRs.getString("COLUMN_NAME")      ); } catch (SQLException sqle) {}
                    try { c.setDATA_TYPE        (columnRs.getInt   ("DATA_TYPE")        ); } catch (SQLException sqle) {}
                    try { c.setTYPE_NAME        (columnRs.getString("TYPE_NAME")        ); } catch (SQLException sqle) {}
                    try { c.setCOLUMN_SIZE      (columnRs.getInt   ("COLUMN_SIZE")      ); } catch (SQLException sqle) {}
                    try { c.setDECIMAL_DIGITS   (columnRs.getInt   ("DECIMAL_DIGITS")   ); } catch (SQLException sqle) {}
                    try { c.setNUM_PREC_RADIX   (columnRs.getInt   ("NUM_PREC_RADIX")   ); } catch (SQLException sqle) {}
                    try { c.setNULLABLE         (columnRs.getInt   ("NULLABLE")         ); } catch (SQLException sqle) {}
                    try { c.setREMARKS          (columnRs.getString("REMARKS")          ); } catch (SQLException sqle) {}
                    try { c.setCOLUMN_DEF       (columnRs.getString("COLUMN_DEF")       ); } catch (SQLException sqle) {}
                    try { c.setCHAR_OCTET_LENGTH(columnRs.getInt   ("CHAR_OCTET_LENGTH")); } catch (SQLException sqle) {}
                    try { c.setORDINAL_POSITION (columnRs.getInt   ("ORDINAL_POSITION") ); } catch (SQLException sqle) {}
                    try { c.setIS_NULLABLE      (columnRs.getString("IS_NULLABLE")      ); } catch (SQLException sqle) {}
                    try { c.setSCOPE_CATLOG     (columnRs.getString("SCOPE_CATLOG")     ); } catch (SQLException sqle) {}
                    try { c.setSCOPE_SCHEMA     (columnRs.getString("SCOPE_SCHEMA")     ); } catch (SQLException sqle) {}
                    try { c.setSCOPE_TABLE      (columnRs.getString("SCOPE_TABLE")      ); } catch (SQLException sqle) {}
                    try { c.setSOURCE_DATA_TYPE (columnRs.getShort ("SOURCE_DATA_TYPE") ); } catch (SQLException sqle) {}
                    columns.add(c);
                }
            } catch (SQLException sqle) {
                Log.logger.error(Messages.LOG_MSG_00023, sqle);
            }
            if (columnMap.size() < maxColumnMapSize){
                columnMap.put(key, columns);
            }
            return columns;  
        }
    }
    
    public String makeUrl(DbConnectInfo dbConnectInfo) {
        String url = "";
        switch (dbConnectInfo.getDbtype()) {
        case DBTypes.DB_TYPE_ORACLE:
            url = "jdbc:oracle:thin:@"+dbConnectInfo.getHost()+":"+dbConnectInfo.getPort()+":"+dbConnectInfo.getDbName();
            break;
        case DBTypes.DB_TYPE_MYSQL:
            url = "jdbc:mysql://"+dbConnectInfo.getHost()+":"+dbConnectInfo.getPort()+"/"+dbConnectInfo.getDbName();
            break;
        default:
            Log.logger.error(Messages.LOG_MSG_00003);
            break;
        }
        
        Log.logger.debug("URL : "+url);
        return url;
    }
    
    public DatabaseMetaData getDatabaseMetaData() {
        DatabaseMetaData dbMeta = null;
        if (isConnectionAvailable()==false) {
            reConnect();
        }

        try {
            dbMeta = connection.getMetaData();
        } catch (SQLException sqle) {
            Log.logger.error(Messages.LOG_MSG_00007, sqle);
        }
        
        return dbMeta;
    }
    
    public DatabaseInfo getDatabaseInfo(DatabaseMetaData dbMeta) {
        if (databaseInfo==null) {
            databaseInfo = new DatabaseInfo();
            try { databaseInfo.setAllProceduresAreCallable(dbMeta.allProceduresAreCallable());                           } catch (Exception e) {}
            try { databaseInfo.setAllTablesAreSelectable(dbMeta.allTablesAreSelectable());                               } catch (Exception e) {}
            try { databaseInfo.setDataDefinitionCausesTransactionCommit(dbMeta.dataDefinitionCausesTransactionCommit()); } catch (Exception e) {}
            try { databaseInfo.setDataDefinitionIgnoredInTransactions(dbMeta.dataDefinitionIgnoredInTransactions());     } catch (Exception e) {}
            try { databaseInfo.setMaxRowSizeIncludeBlobs(dbMeta.doesMaxRowSizeIncludeBlobs());                       } catch (Exception e) {}
            try { databaseInfo.setCatalogSeparator(dbMeta.getCatalogSeparator());                                        } catch (Exception e) {}
            try { databaseInfo.setCatalogTerm(dbMeta.getCatalogTerm());                                                  } catch (Exception e) {}
            try { databaseInfo.setDatabaseMajorVersion(dbMeta.getDatabaseMajorVersion());                                } catch (Exception e) {}
            try { databaseInfo.setDatabaseMinorVersion(dbMeta.getDatabaseMinorVersion());                                } catch (Exception e) {}
            try { databaseInfo.setDatabaseProductName(dbMeta.getDatabaseProductName());                                  } catch (Exception e) {}
            try { databaseInfo.setDatabaseProductVersion(dbMeta.getDatabaseProductVersion());                            } catch (Exception e) {}
            try { databaseInfo.setDefaultTransactionIsolation(dbMeta.getDefaultTransactionIsolation());                  } catch (Exception e) {}
            try { databaseInfo.setDriverMajorVersion(dbMeta.getDriverMajorVersion());                                    } catch (Exception e) {}
            try { databaseInfo.setDriverMinorVersion(dbMeta.getDriverMinorVersion());                                    } catch (Exception e) {}
            try { databaseInfo.setDriverName(dbMeta.getDriverName());                                                    } catch (Exception e) {}
            try { databaseInfo.setDriverVersion(dbMeta.getDriverVersion());                                              } catch (Exception e) {}
            try { databaseInfo.setExtraNameCharacters(dbMeta.getExtraNameCharacters());                                  } catch (Exception e) {}
            try { databaseInfo.setIdentifierQuoteString(dbMeta.getIdentifierQuoteString());                              } catch (Exception e) {}
            try { databaseInfo.setJdbcMajorVersion(dbMeta.getJDBCMajorVersion());                                        } catch (Exception e) {}
            try { databaseInfo.setJdbcMinorVersion(dbMeta.getJDBCMinorVersion());                                        } catch (Exception e) {}
            try { databaseInfo.setMaxBinaryLiteralLength(dbMeta.getMaxBinaryLiteralLength());                            } catch (Exception e) {}
            try { databaseInfo.setMaxCatalogNameLength(dbMeta.getMaxCatalogNameLength());                                } catch (Exception e) {}
            try { databaseInfo.setMaxCharLiteralLength(dbMeta.getMaxCharLiteralLength());                                } catch (Exception e) {}
            try { databaseInfo.setMaxColumnNameLength(dbMeta.getMaxColumnNameLength());                                  } catch (Exception e) {}
            try { databaseInfo.setMaxColumnsInGroupBy(dbMeta.getMaxColumnsInGroupBy());                                  } catch (Exception e) {}
            try { databaseInfo.setMaxColumnsInIndex(dbMeta.getMaxColumnsInIndex());                                      } catch (Exception e) {}
            try { databaseInfo.setMaxColumnsInOrderBy(dbMeta.getMaxColumnsInOrderBy());                                  } catch (Exception e) {}
            try { databaseInfo.setMaxColumnsInSelect(dbMeta.getMaxColumnsInSelect());                                    } catch (Exception e) {}
            try { databaseInfo.setMaxColumnsInTable(dbMeta.getMaxColumnsInTable());                                      } catch (Exception e) {}
            try { databaseInfo.setMaxConnections(dbMeta.getMaxConnections());                                            } catch (Exception e) {}
            try { databaseInfo.setMaxCursorNameLength(dbMeta.getMaxCursorNameLength());                                  } catch (Exception e) {}
            try { databaseInfo.setMaxIndexLength(dbMeta.getMaxIndexLength());                                            } catch (Exception e) {}
            try { databaseInfo.setMaxProcedureNameLength(dbMeta.getMaxProcedureNameLength());                            } catch (Exception e) {}
            try { databaseInfo.setMaxRowSize(dbMeta.getMaxRowSize());                                                    } catch (Exception e) {}
            try { databaseInfo.setMaxSchemaNameLength(dbMeta.getMaxSchemaNameLength());                                  } catch (Exception e) {}
            try { databaseInfo.setMaxStatementLength(dbMeta.getMaxStatementLength());                                    } catch (Exception e) {}
            try { databaseInfo.setMaxStatements(dbMeta.getMaxStatements());                                              } catch (Exception e) {}
            try { databaseInfo.setMaxTableNameLength(dbMeta.getMaxTableNameLength());                                    } catch (Exception e) {}
            try { databaseInfo.setMaxUserNameLength(dbMeta.getMaxUserNameLength());                                      } catch (Exception e) {}
            try { databaseInfo.setNumericFunctions(dbMeta.getNumericFunctions());                                        } catch (Exception e) {}
            try { databaseInfo.setProcedureTerm(dbMeta.getProcedureTerm());                                              } catch (Exception e) {}
            try { databaseInfo.setResultSetHoldability(dbMeta.getResultSetHoldability());                                } catch (Exception e) {}
            try { databaseInfo.setSchemaTerm(dbMeta.getSchemaTerm());                                                    } catch (Exception e) {}
            try { databaseInfo.setSearchStringEscape(dbMeta.getSearchStringEscape());                                    } catch (Exception e) {}
            try { databaseInfo.setSQLKeywords(dbMeta.getSQLKeywords());                                                  } catch (Exception e) {}
            try { databaseInfo.setSQLStateType(dbMeta.getSQLStateType());                                                } catch (Exception e) {}
            try { databaseInfo.setStringFunctions(dbMeta.getStringFunctions());                                          } catch (Exception e) {}
            try { databaseInfo.setSystemFunctions(dbMeta.getSystemFunctions());                                          } catch (Exception e) {}
            try { databaseInfo.setTimeDateFunctions(dbMeta.getTimeDateFunctions());                                      } catch (Exception e) {}
            try { databaseInfo.setUrl(dbMeta.getURL());                                                                  } catch (Exception e) {}
            try { databaseInfo.setUserName(dbMeta.getUserName());                                                        } catch (Exception e) {}
            try { databaseInfo.setCatalogAtStart(dbMeta.isCatalogAtStart());                                             } catch (Exception e) {}
            try { databaseInfo.setReadOnly(dbMeta.isReadOnly());                                                         } catch (Exception e) {}
            try { databaseInfo.setLocatorsUpdateCopy(dbMeta.locatorsUpdateCopy());                                       } catch (Exception e) {}
            try { databaseInfo.setNullPlusNonNullIsNull(dbMeta.nullPlusNonNullIsNull());                                 } catch (Exception e) {}
            try { databaseInfo.setNullsAreSortedAtEnd(dbMeta.nullsAreSortedAtEnd());                                     } catch (Exception e) {}
        }
       
        return databaseInfo;
    }

}
